from typing import Any
import base65536
import base64
from numpy import float64, int64
import pyconverter as pc

class NumberSystems:
    def __init__(self):
        pass

    __doc__ = """
        A class of functions to encode and decode between diferent number systems
    """

    def toBase65536(self, val: Any) -> str:
        return base65536.encode(val)
    toBase65536.__doc__ = "Encode val into base65536"
    
    def fromBase65536(self, val: Any) -> str:
        return base65536.decode(val)
    fromBase65536.__doc__ = 'Decode val from base65536 to a string'
    
    def toBase64(self, val: Any) -> str:
        return base64.b64encode(val)
    toBase64.__doc__ = 'Encode val into base64'

    def fromBase64(self, val: str | bytes) -> str:
        return base64.b64decode(val)
    fromBase64.__doc__ = 'Decodes val from base64 to a string'

    def decToHex(self, val: float64 | int64) -> str:
        return pc.doubletohex(val)
    decToHex.__doc__ = 'Encodes a float or int into hexadecimal'

    def hexToDec(self, val: str) -> float64:
        return pc.hextodouble(val)
    hexToDec.__doc__ = 'Decodes a hexadecimal encoded value into a float'
    
    def decToBin(self, val: float64 | int64) -> str:
        return pc.doubletobin(val)
    decToBin.__doc__ = 'Encodes a float or int into binary'

    def binToDec(self, val: str | bytes) -> float64:
        return pc.bintodouble(val)
    binToDec.__doc__ = 'Decodes binary to a float'

    def hexToOct(self, val: str) -> str:
        return pc.hextooct(val)
    hexToOct.__doc__ = 'Encodes a hexadecimal string into an octal string'

    def decToOct(self, val: float64 | int64) -> str:
        return pc.doubletooct(val)
    decToOct.__doc__ = 'Encodes a float or int into an octal string'

    def binToOct(self, val: str | bytes) -> str:
        return pc.bintooct(val)
    binToOct.__doc__ = 'Encodes a binary string into an octal string'



import hashlib as hl
from numpy import int64

class Hasher:
    def __init__(self):
        pass

    __doc__ = '''
        A class of functions to easily run a value through a variety of hash functions
    '''

    def hash_md5(self, e) -> str:
        return hl.md5(str(e).encode()).hexdigest()

    def hash_sha1(self, e) -> str:
        return hl.sha1(str(e).encode()).hexdigest()

    def hash_sha256(self, e) -> str:
        return hl.sha256(str(e).encode()).hexdigest()

    def hash_sha512(self, e) -> str:
        return hl.sha512(str(e).encode()).hexdigest()

    def hash_sha224(self, e) -> str:
        return hl.sha224(str(e).encode()).hexdigest()

    def hash_sha384(self, e) -> str:
        return hl.sha384(str(e).encode()).hexdigest()

    def hash_sha3_512(self, e) -> str:
        return hl.sha3_512(str(e).encode()).hexdigest()

    def hash_sha3_256(self, e) -> str:
        return hl.sha3_256(str(e).encode()).hexdigest()

    def hash_sha3_224(self, e) -> str:
        return hl.sha3_224(str(e).encode()).hexdigest()

    def hash_sha3_384(self, e) -> str:
        return hl.sha3_384(str(e).encode()).hexdigest()

    def hash_shake128(self, e) -> str:
        return hl.shake_128(str(e).encode()).hexdigest()

    def hash_shake256(self, e) -> str:
        return hl.shake_256(str(e).encode()).hexdigest()

    def hash_blake2b(self, e) -> str:
        return hl.blake2b(str(e).encode()).hexdigest()

    def hash_blake2s(self, e) -> str:
        return hl.blake2s(str(e).encode()).hexdigest()

    def hash_x_blake2b(self, e, i: int64) -> str:
        if i <= 1:
            return self.hash_blake2b(e)
        else:
            return self.hash_x_blake2b(e, i-1)
    
    def hash_x_blake2s(self, e, i: int64) -> str:
        if i <= 1:
            return self.hash_blake2s(e)
        else:
            return self.hash_x_blake2s(e, i-1)

    def hash_x_sha512(self, e, i: int64) -> str:
        if i <= 1:
            return self.hash_sha512(e)
        else:
            return self.hash_x_sha512(e, i-1)

    def hash_x_md5(self, e, i: int64) -> str:
        if i <= 1:
            return self.hash_md5(e)
        else:
            return self.hash_x_md5(e, i-1)

    def hash_x_sha3_512(self, e, i: int64) -> str:
        if i <= 1:
            return self.hash_sha3_512(e)
        else:
            return self.hash_x_sha3_512(e, i-1)

    def hash_x_sha1(self, e, i: int64) -> str:
        if i <= 1:
            return self.hash_sha1(e)
        else:
            return self.hash_x_sha1(e, i-1)

    def hash_x_sha384(self, e, i: int64) -> str:
        if i <= 1:
            return self.hash_sha384(e)
        else:
            return self.hash_x_sha384(e, i-1)

    def hash_x_sha224(self, e, i: int64) -> str:
        if i <= 1:
            return self.hash_sha224(e)
        else:
            return self.hash_x_sha224(e, i-1)

    def hash_x_shake256(self, e, i: int64) -> str:
        if i <= 1:
            return self.hash_shake256(e)
        else:
            return self.hash_x_shake256(e, i-1)

    def hash_x_shake128(self, e, i: int64) -> str:
        if i <= 1:
            return self.hash_shake128(e)
        else:
            return self.hash_x_shake128(e, i-1)

    def hash_x_sha3_384(self, e, i: int64) -> str:
        if i <= 1:
            return self.hash_sha3_384(e)
        else:
            return self.hash_x_sha3_384(e, i-1)

    def hash_x_sha3_224(self, e, i: int64) -> str:
        if i <= 1:
            return self.hash_sha3_224(e)
        else:
            return self.hash_x_sha3_224(e, i-1)



from decimal import DivisionByZero
from numpy import float64, int64

class Fraction:

    __doc__ = '''
        A class for a fraction
    '''

    def __init__(self):
        self.den = float64(1.0)
        self.num = float64(0.0)

    def __init__(self, num: float64 | int64, den: float64 | int64):
        if den == 0.0:
            raise DivisionByZero()
        else:
            self.den = den
            self.num = num
    
    def setNum(self, num: float64 | int64) -> None:
        self.num = num

    def setDen(self, den: float64 | int64) -> None:
        if den == 0.0:
            raise DivisionByZero()
        else:
            self.den = den
    
    def getNum(self) -> float64 | int64:
        return self.num

    def getDen(self) -> float64 | int64:
        return self.den
    
    def eval(self) -> float64:
        if self.den == 0.0:
            raise DivisionByZero()
        else:
            return float64(self.num/self.den)
    eval.__doc__ = 'a function to evaluate the fraction and return a float value'

    def __str__(self) -> str:
        return f"{self.num} / {self.den}"



class FileHelper:

    __doc__ = '''
        A class of functions to assist in reading from files
    '''

    def __init__(self):
        pass

    def readFile(self, path: str) -> str:
        temp = None
        with open(path, 'r') as f:
            temp = f.read()
        return temp
    readFile.__doc__ = 'A function to read an entire file into a string'

    def readListFile(self, path: str, sep: str) -> list:
        temp = None
        with open(path, "r") as f:
            temp = f.read().split(sep)
        return temp
    readListFile.__doc__ = 'A function to read sections of a file seperated by \'sep\' into a list'
    
    def readcsv(self, path: str) -> list:
        list = []
        with open(path, 'r') as f:
            list = f.readlines()
        out = []
        for i in list:
            out.append(i.split(','))
        return out
    readcsv.__doc__ = 'A function to read a csv file into a list of lists'